@@ -2,7 +2,7 @@
 
     Main file for  devfsd  (devfs daemon for Linux).
 
-    Copyright (C) 1998-2000  Richard Gooch
+    Copyright (C) 1998-2001  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -162,11 +162,19 @@
     Updated by      Richard Gooch   10-JUN-2000: Moved version string to
   version.h
 
-    Last updated by Richard Gooch   3-JUL-2000: Added "-C /etc/modules.devfs"
+    Updated by      Richard Gooch   3-JUL-2000: Added "-C /etc/modules.devfs"
   when calling modprobe(8). Fail if a configuration line has EXECUTE modprobe.
 
+    Last updated by Richard Gooch   4-JUL-2000: Added #define __USE_GNU. Thanks
+  to Adam J. Richter <adam@yggdrasil.com>.
+
+    Last updated by Richard Gooch   5-FEB-2001: Switched from __USE_GNU to
+  _GNU_SOURCE, according to recommendation from Ulrich. Do dummy opens of
+  /dev/null so that fds [0:2] are open (and thus may be safely closed later).
+
 
 */
+#define _GNU_SOURCE
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -196,6 +204,9 @@
 #include "devfsd.h"
 #include "version.h"
 
+#ifndef RTLD_DEFAULT  /*  Glibc 2.2 broke old code  */
+#  define RTLD_DEFAULT RTLD_NEXT
+#endif
 
 #define CONFIG_FILE "/etc/devfsd.conf"
 #define MAX_ARGS (6 + 1)
@@ -410,6 +421,10 @@ int main (int argc, char **argv)
 		 mount_point, ERRSTRING);
 	exit (1);
     }
+    /*  Open dummy stdin, stdout and stderr if required, so that closing in
+	do_open_syslog() doesn't close other files  */
+    while ( ( fd = open ("/dev/null", O_RDWR, 0) ) < 2 );
+    close (fd);
     if ( ( fd = open (".devfsd", O_RDONLY, 0) ) < 0 )
     {
 	fprintf (stderr, "Error opening file: \".devfsd\"\t%s\n", ERRSTRING);
@@ -1391,7 +1406,7 @@ static void free_config ()
     {
 	next = so_entry->next;
 	free ( (char *) so_entry->name );
-	if (so_entry->handle != RTLD_NEXT) dlclose (so_entry->handle);
+	if (so_entry->handle != RTLD_DEFAULT) dlclose (so_entry->handle);
 	free (so_entry);
     }
     first_so = NULL;
@@ -1707,7 +1722,7 @@ static struct shared_object *get_shared_object (CONST char *name)
 	free (so);
 	return (NULL);
     }
-    if (strcmp (name, "GLOBAL") == 0) so->handle = RTLD_NEXT;
+    if (strcmp (name, "GLOBAL") == 0) so->handle = RTLD_DEFAULT;
     else if ( ( so->handle = dlopen (name, RTLD_NOW) ) == NULL )
     {
 	free ( (char *) so->name );
